package com.simuwang.manage.service.competition;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.simuwang.base.mapper.RzZhzq2ResultMapper;
import com.simuwang.base.pojo.dto.CompetitionBaseResultDTO;
import com.simuwang.base.pojo.dto.ZhzqCompetitionResultDTO;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ZhzqCompetitionResult extends AbstractCompetitionResult {

    private static final Map<Integer, String> STRATEGY_ID_NAME_MAP = MapUtil.newHashMap();

    static {
        STRATEGY_ID_NAME_MAP.put(1, "主观多头策略");
        STRATEGY_ID_NAME_MAP.put(2, "股票量化策略");
        STRATEGY_ID_NAME_MAP.put(3, "相对价值策略");
        STRATEGY_ID_NAME_MAP.put(4, "CTA策略");
        STRATEGY_ID_NAME_MAP.put(5, "复合策略");
    }

    private final RzZhzq2ResultMapper zhzqResultMapper;

    public ZhzqCompetitionResult(RzZhzq2ResultMapper zhzqResultMapper) {
        this.zhzqResultMapper = zhzqResultMapper;
    }

    @Override
    public boolean isSupport(Integer competitionId) {
        return competitionId != null && competitionId == 49;
    }

    @Override
    protected List<? extends CompetitionBaseResultDTO> getCompetitionResult() {
        List<ZhzqCompetitionResultDTO> competitionResultDTOList = zhzqResultMapper.getResult();
        if (CollUtil.isEmpty(competitionResultDTOList)) {
            return CollUtil.newArrayList();
        }
        // 策略名称排名
        Map<Integer, List<ZhzqCompetitionResultDTO>> strategyResultDTOList = competitionResultDTOList.stream().collect(Collectors.groupingBy(CompetitionBaseResultDTO::getStrategy));
        for (Map.Entry<Integer, List<ZhzqCompetitionResultDTO>> resultEntry : strategyResultDTOList.entrySet()) {
            Integer strategy = resultEntry.getKey();
            String strategyName = STRATEGY_ID_NAME_MAP.get(strategy);
            List<ZhzqCompetitionResultDTO> resultDTOList = resultEntry.getValue();

            // 按照分数降序排序
            resultDTOList.sort(Comparator.comparing(CompetitionBaseResultDTO::getScore, Comparator.nullsFirst(Comparator.naturalOrder())).reversed());
            // 计算排名并设置
            int rank = 1;
            for (int i = 0; i < resultDTOList.size(); i++) {
                CompetitionBaseResultDTO dto = resultDTOList.get(i);
                dto.setRank(rank);
                dto.setStrategyName(strategyName);
                rank = i + 1;
            }
        }
        // 策略分组和排名降序输出
        return competitionResultDTOList.stream()
                .sorted(Comparator.comparing(CompetitionBaseResultDTO::getStrategyName).reversed().thenComparing(CompetitionBaseResultDTO::getRank))
                .collect(Collectors.toList());
    }

}
